// $Id: ActionFrame.cpp,v 1.51 2012/01/04 10:51:14 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Application.h"
#include "ActionFrame.h"
#include "BaseIcons.h"
#include "Command.h"
#include "Debug.h"
#include "File.h"
#include "FileList.h"
#include "FileRecordModel.h"
#include "IconEngine.h"
#include "IncludeFrame.h"
#include "IOString.h"
#include "LogFrame.h"
#include "LogWidget.h"
#include "Menu.h"
#include "QtUtil.h"
#include "InformationDialog.h"
#include "Singleton.h"
#include "StatusBar.h"
#include "TextFormat.h"
#include "TexString.h"
#include "MainWindow.h"
#include "XmlOptions.h"

#include <QtCore/QFile>
#include <QtCore/QTextStream>

//_______________________________________________
const QString ActionFrame::checkBoxNames_[checkBoxCount] =
{
    "Latex",
    "PdfLatex",
    "Bibtex",
    "Dvi to PS",
    "Dvi to PDF",
    "PS to PDF",
    "Check",
    "Clean"
};

//_______________________________________________
ActionFrame::ActionFrame( QWidget* parent ):
    QGroupBox( parent ),
    Counter( "ActionFrame" ),
    citationRegExp_( "\\\\cite\\{+(.+)\\}" ),
    process_( 0 )
{

    Debug::Throw( "ActionFrame::ActrionFrame.\n" );
    setLayout( new QVBoxLayout() );
    layout()->setMargin( 5 );
    layout()->setSpacing( 2 );

    // tooltips
    const char *tooltips[checkBoxCount] =
    {
        "Run Latex (twice)",
        "Run PdfLatex (twice)",
        "Run Bibtex",
        "Convert dvi file into postscript",
        "Convert dvi file into pdf",
        "Convert postscript file into pdf",
        "Check citations, labels, references",
        "Remove latex log files"
    };

    // create checkboxes
    box_ = new QWidget( this );
    layout()->addWidget( box_ );

    QVBoxLayout* checkboxLayout( new QVBoxLayout() );
    checkboxLayout->setSpacing( 2 );
    checkboxLayout->setMargin( 0 );
    box_->setLayout( checkboxLayout );

    unsigned int action_state( XmlOptions::get().get<unsigned int>( "ACTIONS" ) );
    for( unsigned int i=0; i < checkBoxCount; i++ )
    {
        checkbox_.push_back( new QCheckBox( checkBoxNames_[i], box_ ) );
        checkbox_[i]->setToolTip( tooltips[i] );
        checkboxLayout->addWidget( checkbox_[i] );
        if( action_state & (1<<i) ) checkbox_[i]->setChecked( true );
    }

    // run button
    button_ = new QPushButton(  "&Run", this );
    button_->setIcon( IconEngine::get( ICONS::DIALOG_ACCEPT ) );
    layout()->addWidget( button_ );
    button_->setToolTip( "Run/abort selected actions" );
    connect( button_, SIGNAL( clicked() ), SLOT( _buttonPressed() ) );
    state_ = IDLE;

    // connections
    citationRegExp_.setMinimal( true );

}

//_______________________________________________
void ActionFrame::_setState( const ActionFrame::State& state )
{

    Debug::Throw() << "ActionFrame::_setState - " << state << endl;

    // mainwindow
    MainWindow &mainwindow( *static_cast<MainWindow*>(window()) );

    if( state == RUNNING )
    {

        // change button text and status
        button_->setText( "&Abort" );
        button_->setIcon( IconEngine::get( ICONS::DIALOG_CANCEL ) );

        // clear logframe
        mainwindow.logFrame().editor().clear();

        // disable widgets
        box_->setEnabled( false );
        mainwindow.editor().setEnabled(false);
        Singleton::get().application<Application>()->busy();

    } else {

        // clear actions
        actions_.clear();

        // change button text and status
        button_->setText( "&Run" );
        button_->setIcon( IconEngine::get( ICONS::DIALOG_ACCEPT ) );

        // enable widgets
        box_->setEnabled( true );
        mainwindow.editor().setEnabled(true);

        // update state bar
        mainwindow.statusBar().label().clear();
        Singleton::get().application<Application>()->idle();

    }

    // update state
    state_ = state;

}

//_______________________________________________
void ActionFrame::_buttonPressed( void )
{

    // mainwindow
    MainWindow &mainwindow( *static_cast<MainWindow*>(window()) );

    if( state_ == RUNNING )
    {

        // write to log
        LogWidget& log( mainwindow.logFrame().editor() );
        log.append( "\n" );
        log.append( "Abort\n", FORMAT::BOLD );
        actions_.clear();
        _setState( IDLE );

        // kill current process
        if( _process().state() != QProcess::NotRunning )
        { _process().kill(); }

    } else {

        // fill actions depending on CheckBoxes
        // latex
        if( _isChecked( LATEX ) )
        {

            // update include and figure frames
            mainwindow.includeFrame().update();

            // update open previous menu
            Singleton::get().application<Application>()->recentFiles().add( mainwindow.file().truncatedName() );

            // update actions
            actions_.push_back( LATEX );

        }

        // pdf latex
        if( _isChecked( PDFLATEX ) ) {

            // update include and figure frames
            mainwindow.includeFrame().update();

            // update open previous menu
            Singleton::get().application<Application>()->recentFiles().add( mainwindow.file().truncatedName() );

            // update actions
            actions_.push_back( PDFLATEX );

        }

        // bibtex
        if( _isChecked( BIBTEX ) )
        {
            // need to run bibtex 3 time to have references right
            actions_.push_back( BIBTEX );
            actions_.push_back( BIBTEX );
            actions_.push_back( BIBTEX );
            if( _isChecked( LATEX ) )
            {

                actions_.push_back( LATEX );
                actions_.push_back( LATEX );

            } else if( _isChecked( PDFLATEX ) ) {

                actions_.push_back( PDFLATEX );
                actions_.push_back( PDFLATEX );

            }

        } else if( _isChecked( LATEX ) ) {

            // run latex a second time
            actions_.push_back( LATEX );

        } else if( _isChecked( PDFLATEX ) ) {

            // run latex a second time
            actions_.push_back( PDFLATEX );

        }

        // dvips
        if( _isChecked( DVIPS ) ) actions_.push_back( DVIPS );

        // dvipdf
        if( _isChecked( DVIPDF ) ) actions_.push_back( DVIPDF );

        // ps2pdf
        if( _isChecked( PS2PDF ) ) actions_.push_back( PS2PDF );

        // check
        if( _isChecked( CHECK ) ) actions_.push_back( CHECK );

        // clean
        if( _isChecked( CLEAN ) ) actions_.push_back( CLEAN );

        if( actions_.size() ) {

            _setState( RUNNING );
            _updateActionState();
            _run();

            // uniconify log
            QtUtil::uniconify( &mainwindow.logFrame() );

        } else InformationDialog( this, "No action taken." ).exec();

    }
    return;

}

//_______________________________________________
void ActionFrame::_run( int code, QProcess::ExitStatus status )
{

    Debug::Throw( "ActionFrame::_run.\n" );

    // mainwindow
    MainWindow &mainwindow( *static_cast<MainWindow*>(window()) );

    // check status
    if( status != QProcess::NormalExit )
    {
        QString buffer;
        QTextStream( &buffer ) << "process excited with code " << code << endl;
        LogWidget& log( mainwindow.logFrame().editor() );
        log.append( "\n" );
        log.append( buffer, FORMAT::BOLD|FORMAT::UNDERLINE );
    }

    // check if other actions are to be run
    if( actions_.empty() )
    {
        _setState( IDLE );
        return;
    }

    Type type( actions_.front() );
    QStringList command;
    _newProcess();
    switch( type )
    {

        case LATEX:
        command = _latexCommand();
        break;

        case PDFLATEX:
        command = _pdfLatexCommand();
        break;

        case BIBTEX:
        command = _bibtexCommand();
        break;

        case   DVIPS:
        command = _dviPsCommand();
        break;

        case   DVIPDF:
        command = _dviPdfCommand();
        break;

        case PS2PDF:
        command = _psToPdfCommand();
        break;

        case CHECK:
        {
            _checkCitations();
            _checkCitationOrder();
            _checkLabels();
            _checkReferences();
            actions_.pop_front();
            _run();
            return;
        }

        case CLEAN:
        {
            mainwindow.clean();
            actions_.pop_front();
            _run();
            return;
        }

        default:
        {
            QString buffer;
            QTextStream( &buffer ) << checkBoxNames_[ type ] << " not implemented.";
            InformationDialog( this, buffer ).exec();
            break;
        }

    }

    // check process
    if( command.empty() ) {
        Debug::Throw( "ActionFrame::_run - no command.\n" );
        _setState( IDLE );
        return;
    }

    if( Debug::level() )
    {
        Debug::Throw(0) << "ActionFrame::_run - command: ";
        for( QStringList::iterator iter = command.begin(); iter != command.end(); ++iter )
        { Debug::Throw(0) << *iter << " "; }
        Debug::Throw(0) << endl;
    }

    // remove action being processed from list
    actions_.pop_front();

    // update state bar
    QString buffer;
    QTextStream( &buffer ) << "Running " << checkBoxNames_[type];
    mainwindow.statusBar().label().setText( buffer, true );

    LogWidget& log( mainwindow.logFrame().editor() );
    log.append( "\n" );
    log.append( buffer, FORMAT::BOLD|FORMAT::UNDERLINE );
    log.append( "\n" );

    // dump full command
    buffer.clear();
    QTextStream stream( &buffer );
    for( QStringList::iterator iter = command.begin(); iter != command.end(); ++iter )
    { stream << "\"" << *iter << "\" "; }
    log.append( buffer );
    log.append( "\n" );

    // try run
    _process().start( command );

    return;

}

//_______________________________________________
void ActionFrame::_error( QProcess::ProcessError error )
{

    Debug::Throw( "ActionFrame::_error.\n" );
    QString buffer;
    QTextStream( &buffer ) << CustomProcess::errorMessage( error ) << endl;
    static_cast<MainWindow*>(window())->logFrame().editor().append( buffer, FORMAT::BOLD );

    // try next action
    _run();

}

//_______________________________________________
void ActionFrame::_read( void )
{ static_cast<MainWindow*>(window())->logFrame().editor().append( IOString( _process() ) ); }

//_______________________________________________
void ActionFrame::_updateActionState( void )
{
    // create checkboxes
    unsigned int action_state( 0 );
    for( unsigned int i=0; i < checkBoxCount; i++ )
    { action_state |= (checkbox_[i]->isChecked()<<i); }

    XmlOptions::get().set<unsigned int>("ACTIONS", action_state );
    XmlOptions::write();
}

//_______________________________________________
QStringList ActionFrame::_latexCommand( void )
{

    Debug::Throw( "ActionFrame::_latexCommand.\n" );

    // create/check tex filename
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    File file( TexString( mainwindow.file() ).addExtension( ".tex" ) );
    if( !mainwindow.checkFile( file ) ) return QStringList();

    // define process
    _process().setWorkingDirectory( file.path() );
    return  Command( XmlOptions::get().raw( "LATEX_COMMAND" ) ) << file.localName();

}

//_______________________________________________
QStringList ActionFrame::_pdfLatexCommand( void )
{

    Debug::Throw( "ActionFrame::_pdfLatexCommand.\n" );

    // create/check tex filename
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    File file( TexString( mainwindow.file() ).addExtension( ".tex" ) );
    if( !mainwindow.checkFile( file ) ) return QStringList();

    // define process
    _process().setWorkingDirectory( file.path() );
    return  Command( XmlOptions::get().raw( "PDF_LATEX" ) ) << file.localName();

}

//_______________________________________________
QStringList ActionFrame::_bibtexCommand( void )
{

    Debug::Throw( "ActionFrame::_bibtexCommand.\n" );

    // create/check tex filename
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    File file = TexString( mainwindow.file() ).addExtension( ".aux" );
    if( !mainwindow.checkFile( file ) ) return QStringList();

    // define process
    _process().setWorkingDirectory( file.path() );
    return Command( XmlOptions::get().raw( "BIBTEX_COMMAND" ) ) << file.truncatedName().localName();

}

//_______________________________________________
QStringList ActionFrame::_dviPsCommand( void )
{

    Debug::Throw( "ActionFrame::_dviPsCommand.\n" );

    // create/check dvi filename
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    File file( TexString( mainwindow.file() ).addExtension( ".dvi" ) );
    if( !mainwindow.checkFile( file ) ) return QStringList();

    // create ps filename
    File ps_file( TexString( file ).addExtension( ".ps" ) );

    // create process
    _process().setWorkingDirectory( file.path() );

    QStringList out( Command( XmlOptions::get().raw( "DVI_TO_PS" ) ) );

    return out << "-o" << ps_file.localName() << file.localName();

}

//_______________________________________________
QStringList ActionFrame::_dviPdfCommand( void )
{

    Debug::Throw( "ActionFrame::_dviPdfCommand.\n" );

    // create/check dvi filename
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    File file( TexString( mainwindow.file() ).addExtension( ".dvi" ) );
    if( !mainwindow.checkFile( file ) ) return QStringList();

    // create pdf filename
    File pdf_file( TexString( file ).addExtension( ".pdf" ) );

    // create process
    _process().setWorkingDirectory( file.path() );

    // build command
    QStringList out( Command( XmlOptions::get().raw( "DVI_TO_PDF" ) ) );
    return out << file.localName() << pdf_file.localName();

}

//_______________________________________________
QStringList ActionFrame::_psToPdfCommand( void )
{

    Debug::Throw( "ActionFrame::psToPdfCommand.\n" );

    // create/check ps filename
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    File file( TexString( mainwindow.file() ).addExtension( ".ps" ) );
    if( !mainwindow.checkFile( file ) ) return QStringList();

    // create process
    _process().setWorkingDirectory( file.path() );
    return Command( XmlOptions::get().raw( "PS_TO_PDF" ) ) << file.localName();
}

//_______________________________________________
void ActionFrame::_checkCitations( void )
{
    Debug::Throw( "ActionFrame::_checkCitations.\n" );

    // retrieve/check log file
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    LogWidget& log( mainwindow.logFrame().editor() );
    QString file( TexString( mainwindow.file() ).addExtension( ".log" ) );
    if( !mainwindow.checkFile( file ) ) return;

    // parse file
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) )
    {
        log.append( "\n" );
        log.append( "Citations:", FORMAT::BOLD|FORMAT::UNDERLINE );

        QString buffer;
        QTextStream( &buffer ) << "could not parse " << file << endl;
        log.append( buffer );
        return;
    }

    bool found( false );
    bool flush( false );
    QString buffer;
    QTextStream stream( &buffer );
    while( !in.atEnd() )
    {
        QString line( in.readLine( 1024 ) );
        if( line.isNull() ) continue;

        if( (!flush) && !( line.contains( "LaTeX Warning" ) && line.contains( "Citation" ) ) )
        { continue; }

        if( flush && line.size() <= 1 )
        {
            flush = false;
            stream << endl;
            continue;
        }

        flush = true;
        stream << line;
        found = true;
    }

    // close file
    in.close();

    // return if none found
    if( !found ) return;

    log.append( "\n" );
    log.append( "Undefined citations:\n", FORMAT::BOLD|FORMAT::UNDERLINE );
    log.append( buffer );

    return;
}

//_______________________________________________
void ActionFrame::_checkLabels( void )
{
    Debug::Throw( "ActionFrame::_checkLabels.\n" );

    // retrieve/check log file
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    LogWidget& log( mainwindow.logFrame().editor() );
    QString file( TexString( mainwindow.file() ).addExtension( ".log" ) );
    if( !mainwindow.checkFile( file ) ) return;

    // parse file
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) )
    {
        log.append( "\n" );
        log.append( "Labels:", FORMAT::BOLD|FORMAT::UNDERLINE );

        QString buffer;
        QTextStream( &buffer ) << "could not parse file " << file << endl;
        log.append( buffer );
        return;
    }

    bool found( false );
    bool flush( false );
    QString buffer;
    QTextStream stream( &buffer );
    while( !in.atEnd() )
    {

        QString line( in.readLine( 1024 ) );
        if( line.isNull() ) continue;

        if( (!flush) && !( line.contains( "LaTeX Warning" ) && line.contains( "Label" ) ) )
        { continue; }

        if( flush && line.size() <= 1 )
        {
            flush = false;
            stream << endl;
            continue;
        }

        flush = true;
        stream << line;
        found = true;
    }

    // close file
    in.close();

    // return if none found
    if( !found ) return;

    log.append( "\n" );
    log.append( "Label warnings:\n", FORMAT::BOLD|FORMAT::UNDERLINE );
    log.append( buffer );
    log.append( "\n" );

    return;
}

//_______________________________________________
void ActionFrame::_checkReferences( void )
{
    Debug::Throw( "ActionFrame::_checkReferences.\n" );

    // retrieve/check log file
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );
    LogWidget& log( mainwindow.logFrame().editor() );
    QString file( TexString( mainwindow.file() ).addExtension( ".log" ) );
    if( !mainwindow.checkFile( file ) ) return;

    // parse file
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) )
    {
        log.append( "\n" );
        log.append( "References:", FORMAT::BOLD|FORMAT::UNDERLINE );

        QString buffer;
        QTextStream( &buffer ) << "could not parse file " << file << "" << endl;
        log.append( buffer );
        return;
    }


    bool found( false );
    bool flush( false );
    QString buffer;
    QTextStream stream( &buffer );
    while( !in.atEnd() )
    {
        QString line( in.readLine( 1024 ) );
        if( line.isNull() ) continue;
        if( (!flush) && !(line.contains( "LaTeX Warning" ) && line.contains("Reference" ) ) )
        { continue; }

        if( flush && line.size() <= 1 )
        {
            flush = false;
            stream << endl;
            continue;
        }

        flush = true;
        stream << line;
        found = true;
    }

    // close file
    in.close();

    // return if none found
    if( !found ) return;

    log.append( "\n" );
    log.append( "Undefined references:\n", FORMAT::BOLD|FORMAT::UNDERLINE );
    log.append( buffer );
    log.append( "\n" );
    return;
}

//_____________________________________________________
void ActionFrame::_checkCitationOrder( void )
{

    Debug::Throw( "ActionFrame::_checkCitationOrder.\n" );

    // retrieve/check log file
    MainWindow& mainwindow( *static_cast<MainWindow*>(window()) );

    FileRecordModel::List allRecords;
    IncludeFileModel::List includeFiles( mainwindow.includeFrame().files() );
    std::list<QString> citations;
    for( IncludeFileModel::List::const_iterator iter = includeFiles.begin(); iter != includeFiles.end(); ++iter )
    {
        if( !mainwindow.checkFile( iter->file(), true ) ) continue;
        _addCitations( citations, iter->file() );

    }

    QString buffer;
    QTextStream stream( &buffer );
    for( std::list<QString>::iterator iter = citations.begin(); iter != citations.end(); ++iter )
    { stream << *iter << endl; }

    // append to log
    LogWidget& log( mainwindow.logFrame().editor() );
    log.append( "\n" );
    log.append( "Citation order:\n", FORMAT::BOLD|FORMAT::UNDERLINE );
    log.append( buffer );
    log.append( "\n" );

    return;

}

//_____________________________________________________
void ActionFrame::_addCitations( std::list<QString>& citation_list, const File& file )
{

    Debug::Throw( "ActionFrame::_addCitations.\n" );

    // try open file
    File fullname( file.expand() );
    QFile in( fullname );
    if( !in.open( QIODevice::ReadOnly ) ) return;

    // retrieve text
    QString text( in.readAll() );

    // split by lines
    QStringList lines( text.split( "\n" ) );

    for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
    {

        // get position of regexp in line.
        int position( 0 );
        while( ( position = citationRegExp_.indexIn( *iter, position ) ) >= 0 )
        {


            // split using "," as a separator in case of multiple citations
            QStringList citations( citationRegExp_.cap(1).split( "," ) );
            for( QStringList::const_iterator citationIter = citations.begin(); citationIter != citations.end(); ++citationIter )
            {
                QString citation( *citationIter );
                if( std::find( citation_list.begin(), citation_list.end(), citation ) == citation_list.end() ) citation_list.push_back( citation );
            }

            position += citationRegExp_.matchedLength();

        }

    }

    return;

}

//___________________________________________________________
CustomProcess& ActionFrame::_newProcess( void )
{
    // delete old process
    if( process_ )
    {
        if( _process().state() != QProcess::NotRunning ) _process().kill();
        _process().deleteLater();
    }

    // create new one
    process_ = new CustomProcess( this );
    process_->setReadChannelMode(QProcess::MergedChannels);
    connect( &_process(), SIGNAL( readyRead() ), SLOT( _read() ) );
    connect( &_process(), SIGNAL( error( QProcess::ProcessError ) ), SLOT( _error( QProcess::ProcessError ) ) );
    connect( &_process(), SIGNAL( finished( int, QProcess::ExitStatus ) ), SLOT( _run( int, QProcess::ExitStatus ) ) );
    return _process();

}
