// $Id: Application.cpp,v 1.28 2011/12/12 15:32:32 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file Application.cpp
\brief application Main Window singleton object
\author Hugo Pereira
\version $Revision: 1.28 $
\date $Date: 2011/12/12 15:32:32 $
*/

#include <QApplication>
#include <QMessageBox>
#include <QPixmap>
#include <QTextStream>

#include "BaseIcons.h"
#include "Config.h"
#include "ConfigurationDialog.h"
#include "Debug.h"
#include "IconEngine.h"
#include "Application.h"
#include "Menu.h"
#include "RecentFilesMenu.h"
#include "QtUtil.h"
#include "QuestionDialog.h"
#include "MainWindow.h"
#include "XmlFileList.h"
#include "XmlOptions.h"

//____________________________________________
void Application::usage( void )
{

    Debug::Throw(0) << "Usage : textool [options] [file]" << endl;
    SERVER::ApplicationManager::commandLineParser().usage();
    return;

}

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( 0, arguments  ),
    Counter( "Application" ),
    MAIN_TITLE( "latex tools" ),
    recentFiles_( 0 ),
    mainWindow_(0)
{}

//____________________________________________
Application::~Application( void )
{
    Debug::Throw( "Application::~Application.\n" );
    if( recentFiles_ ) delete recentFiles_;
}

//____________________________________________
void Application::initApplicationManager( void )
{
    Debug::Throw( "Application::initApplicationManager.\n" );

    // retrieve files from arguments and expand if needed
    CommandLineParser parser( SERVER::ApplicationManager::commandLineParser( _arguments() ) );
    QStringList& orphans( parser.orphans() );
    for( QStringList::iterator iter = orphans.begin(); iter != orphans.end(); ++iter )
    { if( !iter->isEmpty() ) (*iter) = File( *iter ).expand(); }

    // replace arguments
    _setArguments( parser.arguments() );

    // base class initialization
    BaseApplication::initApplicationManager();

}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::RealizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // rename about action
    aboutAction().setText( "About &textool" );
    configurationAction().setText( "Configure &textool ..." );

    // file list
    recentFiles_ = new XmlFileList();
    recentFiles_->setCheck( false );

    // set icon Pixmap
    mainWindow_ = new MainWindow(0);

    // configuration
    emit configurationChanged();

    mainWindow().centerOnDesktop();
    mainWindow().show();

    QStringList filenames( SERVER::ApplicationManager::commandLineParser( _arguments() ).orphans() );
    if( !filenames.isEmpty() ) mainWindow().open( FileRecord( File( filenames.front() ).expand() ) );
    else mainWindow().open( recentFiles().lastValidFile() );

    return true;

}

//_________________________________________________
void Application::_configuration( void )
{

    Debug::Throw( "Application::configuration.\n" );
    emit saveConfiguration();

    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL( configurationChanged() ), SIGNAL( configurationChanged() ) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//________________________________________________
bool Application::_processCommand( SERVER::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == SERVER::ServerCommand::RAISE )
    {
        if( mainWindow_ ) mainWindow().uniconify();

        QStringList filenames( SERVER::ApplicationManager::commandLineParser( command.arguments() ).orphans() );
        if( !filenames.isEmpty() )
        {
            QString buffer;
            QTextStream( &buffer ) << "Accept request for file \"" << filenames.front() << "\" ?";
            if( QuestionDialog( mainWindow_, buffer ).centerOnParent().exec() )
            { mainWindow().open( File( filenames.front() ) ); }
        }

        return true;

    } else return false;

}
