// $Id: ButtonFrame.cpp,v 1.35 2011/12/11 11:17:08 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "ButtonFrame.h"

#include "Application.h"
#include "Command.h"
#include "LineEditor.h"
#include "Debug.h"
#include "File.h"
#include "FileList.h"
#include "IncludeFrame.h"
#include "LogFrame.h"
#include "Menu.h"
#include "InformationDialog.h"
#include "RecentFilesMenu.h"
#include "Singleton.h"
#include "StatusBar.h"
#include "TexString.h"
#include "MainWindow.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QtGui/QPushButton>
#include <QtGui/QToolTip>
#include <QtGui/QLayout>

//_______________________________________________________
ButtonFrame::ButtonFrame( QWidget *parent ):
QWidget( parent ),
Counter( "ButtonFrame" )
{

    Debug::Throw( "ButtonFrame::ButtonFrame" );
    QVBoxLayout* layout( new QVBoxLayout() );
    setLayout( layout );
    layout->setMargin( 0 );
    layout->setSpacing( 2 );

    // edit tex file
    layout->addWidget( editButton_ = new QPushButton( "&Edit Tex", this ) );
    editButton_->setToolTip( "Edit tex file" );

    // generic button
    QPushButton* button;

    // XDVI
    layout->addWidget( button = new QPushButton( "&View DVI", this ) );
    connect( button, SIGNAL( clicked() ), SLOT( _dviViewer() ) );
    button->setToolTip( "View dvi file" );

    // View PS
    layout->addWidget( button = new QPushButton( "View &PS", this ) );
    connect( button, SIGNAL( clicked() ), SLOT( _psViewer() ) );
    button->setToolTip( "View postscript file" );

    // View PDF
    layout->addWidget( button = new QPushButton( "Vie&w PDF", this ) );
    connect( button, SIGNAL( clicked() ), SLOT( _pdfViewer() ) );
    button->setToolTip( "View pdf file" );

    // spell check
    layout->addWidget( button = new QPushButton( "Spe&llcheck", this ) );
    connect( button, SIGNAL( clicked() ), SLOT( _spellCheck() ) );
    button->setToolTip( "External spellchecker" );

    // clean
    layout->addWidget( button = new QPushButton( "&Clean", this ) );
    connect( button, SIGNAL( clicked() ), window(), SLOT( clean() ) );
    button->setToolTip( "Delete latex temporary files" );

    layout->addStretch(1);

}


//_____________________________________
void ButtonFrame::_dviViewer( void )
{
    Debug::Throw( "ButtonFrame::_dviViewer.\n" );

    // check latex command
    if( !XmlOptions::get().find( "DVI_VIEWER" ) ) {
        InformationDialog( this, "No dvi viewer set." ).exec();
        return;
    }

    Command dvi_viewer( XmlOptions::get().raw( "DVI_VIEWER" ) );

    // create/check dvi filename
    Application& application( *Singleton::get().application<Application>() );
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    File file( TexString( mainWindow.file() ).addExtension( ".dvi" ) );
    if( !mainWindow.checkFile( file ) ) return;

    // run and update
    application.busy();
    QString buffer;
    QTextStream( &buffer ) << dvi_viewer.join( " " ) << " ...";
    mainWindow.statusBar().label().setText( buffer );
    ( dvi_viewer << file ).run( file.path() );

    application.idle();
    mainWindow.statusBar().label().clear();
    mainWindow.setFile( file );
    Singleton::get().application<Application>()->recentFiles().add( file.truncatedName() );
    mainWindow.menu().recentFilesMenu().setCurrentFile( file.truncatedName() );

    return;
}

//_____________________________________
void ButtonFrame::_psViewer( void )
{
    Debug::Throw( "ButtonFrame::_psViewer.\n" );

    // check ps_viewer command
    if( !XmlOptions::get().find( "PS_VIEWER" ) ) {
        InformationDialog( this, "PS viewer not set." ).exec();
        return;
    }

    Command ps_viewer( XmlOptions::get().raw("PS_VIEWER") );

    // create/check ps filename
    Application& application( *Singleton::get().application<Application>() );
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    File file( TexString( mainWindow.file() ).addExtension( ".ps" ) );
    if( !mainWindow.checkFile( file ) ) return;

    // run and update
    application.busy();
    QString buffer;
    QTextStream( &buffer ) << "Running " << ps_viewer.join( " " ) << " ...";
    mainWindow.statusBar().label().setText( buffer );
    ( ps_viewer << file ).run( file.path() );

    application.idle();
    mainWindow.statusBar().label().clear();
    mainWindow.setFile( file );
    Singleton::get().application<Application>()->recentFiles().add( file.truncatedName() );
    mainWindow.menu().recentFilesMenu().setCurrentFile( file.truncatedName() );

    return;
}

//_____________________________________
void ButtonFrame::_pdfViewer( void )
{
    Debug::Throw( "ButtonFrame::_RunPDFViewer.\n" );


    // create/check pdf filename
    Application& application( *Singleton::get().application<Application>() );
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    File file( TexString( mainWindow.file() ).addExtension( ".pdf" ) );
    if( !mainWindow.checkFile( file ) ) return;

    // run and update
    application.busy();
    Command pdf_viewer( XmlOptions::get().raw( "PDF_VIEWER" ) );
    QString buffer;
    QTextStream( &buffer ) << "Running " << pdf_viewer.join(" ") << " ...";
    mainWindow.statusBar().label().setText( buffer );
    ( pdf_viewer << file.localName() ).run( file.path() );

    application.idle();
    mainWindow.statusBar().label().clear();
    mainWindow.setFile( file );
    Singleton::get().application<Application>()->recentFiles().add( file.truncatedName() );
    mainWindow.menu().recentFilesMenu().setCurrentFile( file.truncatedName() );

    return;
}

//_____________________________________________________________
void ButtonFrame::_spellCheck(void)
{

    // check spell checker command
    if( !XmlOptions::get().find( "SPELLCHECK" ) ) {
        InformationDialog( this, "spellcheck command not set." ).exec();
        return;
    }

    // see if there are selected items in the Include Frame
    Application& application( *Singleton::get().application<Application>() );
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    IncludeFileModel::List selection( mainWindow.includeFrame().selection() );

    // if list of selected files is empty, add the one from FileSelection frame.
    if( selection.empty() ) selection.push_back( IncludeFile( mainWindow.file() ) );

    // loop over included files
    application.busy();
    for( IncludeFileModel::List::const_iterator iter = selection.begin(); iter != selection.end(); ++iter )
    {
        File file( TexString( iter->file() ).addExtension( ".tex" ) );
        if( !mainWindow.checkFile( file ) ) return;

        QString buffer;
        QTextStream( &buffer ) << "Running " <<  XmlOptions::get().raw( "SPELLCHECK" ) << " ...";
        mainWindow.statusBar().label().setText( buffer );

        ( Command( XmlOptions::get().raw( "SPELLCHECK" ) ) << file ).run( file.path() );

    }

    application.idle();
    mainWindow.statusBar().label().clear();
}
