// $Id: FigureFrame.cpp,v 1.35 2012/01/04 10:51:14 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Application.h"
#include "Command.h"
#include "Debug.h"
#include "FigureFrame.h"
#include "IncludeFrame.h"
#include "MainWindow.h"
#include "Options.h"
#include "InformationDialog.h"
#include "Singleton.h"
#include "StatusBar.h"
#include "TexString.h"
#include "TreeView.h"
#include "Util.h"

#include <QtCore/QFile>
#include <QtGui/QHeaderView>

//_____________________________________________________________
FigureFrame::FigureFrame( QTabWidget* parent ):
    TabWidget( parent ),
    graphicsPathRegExp_( "\\\\graphicspath\\{+((?:\\w|/)+)\\}" ),
    figureRegExp_( "\\\\includegraphics(?:\\[.*\\])?\\{+(.+)\\}" ),
    commentRegExp_( "^(\\s*)%" )
{

    Debug::Throw("FigureFrame::FigureFrame.\n" );
    setTitle( "figures" );

    box().layout()->addWidget( list_ = new TreeView( &box() ) );
    _list().setModel( &model_ );
    _list().setSortingEnabled( false );
    _list().setSelectionMode( QAbstractItemView::ContiguousSelection );
    _list().header()->hide();
    _list().setMask( 1 << FileRecordModel::FILE );

    // add buttons to menu
    viewFigureAction_ = new QAction( "&View Figure", this );
    connect( viewFigureAction_, SIGNAL( triggered() ), SLOT( _viewFigure() ) );
    _list().menu().addAction( viewFigureAction_ );

    // connections
    connect( _list().selectionModel(), SIGNAL( selectionChanged( const QItemSelection& , const QItemSelection& ) ), SLOT( _updateActions() ) );
    connect( &_list(), SIGNAL( activated( const QModelIndex& ) ), viewFigureAction_, SIGNAL( triggered() ) );

    // tweak regexps
    figureRegExp_.setMinimal( true );

}

//_____________________________________________________________
void FigureFrame::update( IncludeFileModel::List includeFiles )
{
    Debug::Throw( "FigureFrame::update.\n" );

    // clear list
    model_.clear();

    // reset Graphics path
    graphicsPath_ = File("");

    FileRecordModel::List allRecords;
    for( IncludeFileModel::List::const_iterator iter = includeFiles.begin(); iter != includeFiles.end(); ++iter )
    {

        // try cast to a File
        const File& file( iter->file() );
        FileRecordModel::List records( _parseFile( file ) );
        if( allRecords.empty() ) allRecords = records;
        else if( !records.empty() )
        {
            for( FileRecordModel::List::const_iterator iter = records.begin(); iter != records.end(); ++iter )
            { allRecords.push_back( *iter ); }
        }

    }

    model_.update( allRecords );

}

//_____________________________________________________________
void FigureFrame::_updateActions( void )
{
    Debug::Throw( "FigureFrame::_updateActions.\n" );
    viewFigureAction_->setEnabled(  !_list().selectionModel()->selectedRows().empty() );
    return;
}

//_____________________________________________________________
void FigureFrame::_viewFigure( void )
{
    Debug::Throw( "FigureFrame::_viewFigure.\n" );

    // retrieve selection
    FileRecordModel::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
    if( selection.empty() )
    {
        InformationDialog( this, "No file selected. <Edit> canceled." ).exec();
        return;
    }

    // check ps_viewer command
    if( !XmlOptions::get().find( "PS_VIEWER" ) )
    {
        InformationDialog( this, "postscript viewer not set." ).exec();
        return;
    }

    Application &application( *Singleton::get().application<Application>() );
    MainWindow &mainWindow( *static_cast<MainWindow*>(parentTabWidget().window()) );
    application.busy();
    for( FileRecordModel::List::const_iterator iter = selection.begin(); iter != selection.end(); ++iter )
    {

        File file( iter->file() );

        // if file does not exists try append ".eps" extension
        if( !file.exists() )
        {

            if( File( file.truncatedName() + ".eps" ).exists() ) file = file.truncatedName() + ".eps";
            else if( File( file + ".eps" ).exists() ) file = file + ".eps";
            else {
                QString buffer;
                QTextStream( &buffer ) << "file \"" << file << "\" does not exists. <ViewFigure> aborted.";
                InformationDialog( this, buffer ).exec();
                continue;
            }

        }

        // check if file is a directory
        if( file.isDirectory() ) {
            QString buffer;
            QTextStream( &buffer ) << "file \"" << file << "\" is a directory. <ViewFigure> aborted.";
            InformationDialog( this, buffer ).exec();
            continue;
        }

        // update display and run command
        QString buffer;
        QTextStream( &buffer ) << "Running " <<  XmlOptions::get().raw( "PS_VIEWER" ) << " ...";
        mainWindow.statusBar().label().setText( buffer );

        ( Command( XmlOptions::get().raw( "PS_VIEWER" ) ) << file.localName() ).run(file.path());

    }

    application.idle();
    mainWindow.statusBar().label().clear();

}

//_____________________________________________________________
FileRecordModel::List FigureFrame::_parseFile( const File& file )
{
    Debug::Throw() << "FigureFrame::_parseFile - file: " << file << endl;

    FileRecordModel::List records;

    // try open file
    File fullname( file.expand() );
    QFile in( fullname );
    if( !in.open( QIODevice::ReadOnly ) ) return records;

    // retrieve text
    QString text( in.readAll() );
    in.close();

    // look for graphics tag
    QStringList lines( text.split( "\n" ) );
    for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
    {

        // check if match
        if( graphicsPathRegExp_.indexIn( *iter ) < 0 ) continue;

        // check if commented
        if( commentRegExp_.indexIn( *iter ) >= 0 ) continue;

        // store graphic path
        graphicsPath_ = File( graphicsPathRegExp_.cap(1) );
        if( !graphicsPath_.isAbsolute() ) graphicsPath_ = graphicsPath_.addPath( file.path() );
        Debug::Throw() << "FigureFrame::_parseFile - graphics_path: " << graphicsPath_ << endl;
    }

    // parse file for figures
    for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
    {

        int position( 0 );
        while( ( position = figureRegExp_.indexIn( *iter, position ) ) >= 0 )
        {

            // check if commented
            if( commentRegExp_.indexIn( *iter ) < 0 )
            {
                File figure( figureRegExp_.cap(1) );
                if( !figure.isAbsolute() ) {
                    if( graphicsPath_.size() ) figure = figure.addPath( graphicsPath_ );
                    else figure = figure.addPath( file.path() );
                }

                // create new item
                Debug::Throw() << "FigureFrame::_parseFile - fullname figure: " << figure << endl;
                records.push_back( FileRecord( figure ) );

            }

            position += figureRegExp_.matchedLength();
        }
    }

    return records;
}
