// $Id: IncludeFrame.cpp,v 1.38 2012/01/04 10:51:14 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\File IncludeFrame.cpp
\brief parse input File and display includes recursively
\author Hugo Pereira
\version $Revision: 1.38 $
\date $Date: 2012/01/04 10:51:14 $
*/

#include <QFile>
#include <QHeaderView>

#include "Application.h"
#include "Command.h"
#include "Debug.h"
#include "IncludeFrame.h"
#include "InformationDialog.h"
#include "MainWindow.h"
#include "StatusBar.h"
#include "Singleton.h"
#include "TexString.h"
#include "TreeView.h"
#include "Util.h"
#include "XmlOptions.h"

//_____________________________________________________________
IncludeFrame::IncludeFrame( QTabWidget* parent ):
    TabWidget( parent ),
    includeRegexp_( "\\\\(input|include|bibliography)\\{(.+)\\}" ),
    commentRegexp_( "^(\\s*)%" )
{
    Debug::Throw("IncludeFrame::IncludeFrame.\n" );
    setTitle( "includes" );

    QBoxLayout* hLayout( static_cast<QBoxLayout*>( box().layout() ) );
    hLayout->setDirection( QBoxLayout::LeftToRight );

    hLayout->addWidget( list_ = new TreeView( &box() ), 1 );
    _list().setModel( &model_ );
    _list().setRootIsDecorated( true );
    _list().setSortingEnabled(false);
    _list().setSelectionMode( QAbstractItemView::ContiguousSelection );
    _list().header()->hide();

    editAction_ = new QAction( "&Edit", this );
    connect( editAction_, SIGNAL( triggered() ), SLOT( _edit() ) );

    spellcheckAction_ = new QAction( "&Spellcheck", this );
    connect( spellcheckAction_, SIGNAL( triggered() ), SLOT( _spellCheck() ) );

    // add buttons to menu
    _list().menu().addAction( editAction_ );
    _list().menu().addAction( spellcheckAction_ );

    // buttons
    QVBoxLayout* vLayout( new QVBoxLayout() );
    vLayout->setMargin(0);
    vLayout->setSpacing(2);
    hLayout->addLayout( vLayout );

    vLayout->addWidget( buttonWidget_ = new QWidget( &box() ), 0 );
    {
        QVBoxLayout* vLayout( new QVBoxLayout() );
        vLayout->setMargin(0);
        vLayout->setSpacing(2);
        buttonWidget_->setLayout( vLayout );

        vLayout->addWidget( selectAllButton_ = new QPushButton( "Select All", buttonWidget_ ) );
        vLayout->addWidget( selectNoneButton_ = new QPushButton( "Select None", buttonWidget_ ) );
        vLayout->addWidget( invertButton_ = new QPushButton( "Invert Selection", buttonWidget_ ) );

        // separator
        QFrame* frame( new QFrame( buttonWidget_ ) );
        frame->setFrameStyle( QFrame::HLine|QFrame::Sunken );
        vLayout->addWidget( frame );

    }

    vLayout->addWidget( editButton_ = new QPushButton( "&Edit", &box() ) );
    vLayout->addWidget( spellCheckButton_ = new QPushButton( "&Spellcheck", &box() ) );

    vLayout->addStretch();

    // connections
    connect( _list().selectionModel(), SIGNAL( selectionChanged( const QItemSelection& , const QItemSelection& ) ), SLOT( _updateActions() ) );
    connect( &_list(), SIGNAL( activated( const QModelIndex& ) ), editAction_, SIGNAL( triggered() ) );
    connect( &_list(), SIGNAL( clicked( const QModelIndex& ) ), SLOT( _toggle( const QModelIndex& ) ) );
    connect( &_list(), SIGNAL( clicked( const QModelIndex& ) ), SLOT( _updateButtons( void ) ) );
    connect( selectAllButton_, SIGNAL( pressed() ), SLOT( _selectAll() ) );
    connect( selectNoneButton_, SIGNAL( pressed() ), SLOT( _selectNone() ) );
    connect( invertButton_, SIGNAL( pressed() ), SLOT( _invert() ) );
    connect( editButton_, SIGNAL( pressed() ), SLOT( _edit() ) );
    connect( spellCheckButton_, SIGNAL( pressed() ), SLOT( _spellCheck() ) );
    connect( &fileSystemWatcher_, SIGNAL( fileChanged( const QString& ) ), SLOT( update( void ) ) );
    buttonWidget_->setVisible( false );
    _updateActions();

    // tweak regular expressions
    includeRegexp_.setMinimal( true );

}

//_____________________________________________________________
bool IncludeFrame::filesModified( void )
{

    Debug::Throw( "IncludeFrame::filesModified.\n" );

    // check last updated
    if( !lastUpdate_.isValid() ) return true;

    // loop over include files
    for( IncludeFileModel::Set::const_iterator iter = includes_.begin(); iter != includes_.end(); ++iter )
    {
        if( !_fileIsAfs( iter->file() ) ) continue;
        if( !iter->file().exists() ) continue;
        if( iter->file().lastModified() > lastUpdate_ ) return true;
    }

    return false;

}

//_____________________________________________________________
IncludeFileModel::List IncludeFrame::selection( void ) const
{ return model_.get( _list().selectionModel()->selectedRows() ); }

//_____________________________________________________________
void IncludeFrame::update( void )
{
    Debug::Throw( "IncludeFrame::Update.\n" );

    // create/check tex Filename
    IncludeFile parent( File( TexString( static_cast<MainWindow*>( parentTabWidget().window() )->file() ).addExtension( ".tex" ) ) );

    // clear model if File is not current
    if( includes_.find( parent ) == includes_.end() )
    {
        model_.clear();
        model_.add( parent );
    }

    // keep track of old files
    const IncludeFileModel::Set oldIncludes( includes_ );

    // clear filesystem watcher
    for( IncludeFileModel::Set::const_iterator iter = includes_.begin(); iter != includes_.end(); ++iter )
    { fileSystemWatcher_.removePath( iter->file() ); }

    // clear includes and add current
    includes_.clear();
    includes_.insert( parent );

    // do nothing if File does not exist
    if( parent.file().exists() )
    {

        // start parsing
        _parseFile( parent );

        // update model
        model_.set( includes_ );

        // check for "new" files and expand parent automatically
        for( IncludeFileModel::Set::const_iterator iter = includes_.begin(); iter != includes_.end(); ++iter )
        {
            if( oldIncludes.find( *iter ) == oldIncludes.end() )
            { _list().expand( model_.parent( model_.index( *iter ) ) ); }
        }

        // update buttons and resize list
        _updateButtons();
        _list().resizeColumns();

    }

    lastUpdate_ = TimeStamp::now();

    // clear filesystem watcher
    for( IncludeFileModel::Set::const_iterator iter = includes_.begin(); iter != includes_.end(); ++iter )
    { if( iter->file().exists() ) fileSystemWatcher_.addPath( iter->file() ); }

    // emit signal
    emit updated( files() );

    return;

}

//_____________________________________________________________
void IncludeFrame::_updateActions( void )
{
    Debug::Throw( "FigureFrame::_updateActions.\n" );
    bool hasSelection( !_list().selectionModel()->selectedRows().empty() );
    editAction_->setEnabled( hasSelection );
    spellcheckAction_->setEnabled( hasSelection );
    return;
}



//_____________________________________________________________
void IncludeFrame::_edit( void )
{

    // get selection
    Debug::Throw( "IncludeFrame::_edit.\n" );

    // retrieve selection
    IncludeFileModel::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
    if( selection.empty() )
    {
        InformationDialog( this, "No File selected. <Edit> canceled." ).exec();
        return;
    }

    // check editor
    if( !XmlOptions::get().find( "TEXT_EDITOR" ) ) {
        InformationDialog( this, "Text editor not set." ).exec();
        return;
    }

    Application &application( *Singleton::get().application<Application>() );
    MainWindow &mainwindow( *static_cast<MainWindow*>(parentTabWidget().window()) );
    application.busy();

    Command command( XmlOptions::get().raw( "TEXT_EDITOR" ) );
    for( IncludeFileModel::List::const_iterator iter = selection.begin(); iter != selection.end(); ++iter )
    {

        const File& file( iter->file() );

        if( !file.exists() )
        {
            QString buffer;
            QTextStream( &buffer ) << "File \"" << file << "\" does not exists. <Edit> aborted.";
            InformationDialog( this, buffer ).exec();
            continue;
        }

        if( file.isDirectory() ) {
            QString buffer;
            QTextStream( &buffer ) << "File \"" << file << "\" is a directory. <Edit> aborted.";
            InformationDialog( this, buffer ).exec();
            continue;
        }

        command << file;

    }

    QString buffer;
    QTextStream( &buffer ) << "Running " << XmlOptions::get().raw("TEXT_EDITOR" ) << " ...";
    mainwindow.statusBar().label().setText( buffer );

    Debug::Throw() << "IncludeFrame::_edit - command: " << command.join( " " ) << endl;
    command.run();

    application.idle();
    mainwindow.statusBar().label().clear();
}

//_____________________________________________________________
void IncludeFrame::_spellCheck(void)
{
    Debug::Throw( "IncludeFrame::_SpellCheck.\n" );

    // retrieve selection
    IncludeFileModel::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
    if( selection.empty() )
    {
        InformationDialog( this, "No File selected. <Edit> canceled." ).exec();
        return;
    }

    // check spell checker command
    if( !XmlOptions::get().find( "SPELLCHECK" ) ) {
        InformationDialog( this, "spellcheck command not set." ).exec();
        return;
    }

    Application &application( *Singleton::get().application<Application>() );
    MainWindow &mainwindow( *static_cast<MainWindow*>(parentTabWidget().window()) );
    application.busy();

    Command command( XmlOptions::get().raw( "SPELLCHECK" ) );
    for( IncludeFileModel::List::const_iterator iter = selection.begin(); iter != selection.end(); ++iter )
    {

        const File& file( iter->file() );
        if( !file.exists() ) {
            QString buffer;
            QTextStream( &buffer ) << "File \"" << file << "\" does not exists. <Edit> aborted.";
            InformationDialog( this, buffer ).exec();
            continue;
        }

        if( file.isDirectory() ) {
            QString buffer;
            QTextStream( &buffer ) << "File \"" << file << "\" is a directory. <Edit> aborted.";
            InformationDialog( this, buffer ).exec();
            continue;
        }

        // update display and run.
        command << file;
    }

    QString buffer;
    QTextStream( &buffer ) << "Running " << XmlOptions::get().raw("SPELLCHECK" ) << " ...";
    mainwindow.statusBar().label().setText( buffer );
    command.run();

    application.idle();
    mainwindow.statusBar().label().clear();

}

//_____________________________________________________________
void IncludeFrame::_selectAll(void)
{
    Debug::Throw( "IncludeFrame::_selectAll.\n" );

    // get all files
    IncludeFileModel::List include_files( model_.children() );

    // loop over files
    for( IncludeFileModel::List::const_iterator iter = include_files.begin(); iter != include_files.end(); ++iter )
    {
        if( iter->parent().file().isEmpty() ) continue;
        if( !iter->isSelected() ) _toggle( model_.index( *iter ) );
    }

    selectAllButton_->setEnabled( false );
    selectNoneButton_->setEnabled( true );

}

//_____________________________________________________________
void IncludeFrame::_selectNone(void)
{
    Debug::Throw( "IncludeFrame::_selectNone.\n" );

    // get all files
    IncludeFileModel::List include_files( model_.children() );

    // loop over files
    for( IncludeFileModel::List::const_iterator iter = include_files.begin(); iter != include_files.end(); ++iter )
    {
        if( iter->parent().file().isEmpty() ) continue;
        if( iter->isSelected() ) _toggle( model_.index( *iter ) );
    }

    selectAllButton_->setEnabled( true );
    selectNoneButton_->setEnabled( false );

}

//_____________________________________________________________
void IncludeFrame::_invert(void)
{
    Debug::Throw( "IncludeFrame::_invert.\n" );

    // get all files
    IncludeFileModel::List include_files( model_.children() );

    // loop over files
    for( IncludeFileModel::List::const_iterator iter = include_files.begin(); iter != include_files.end(); ++iter )
    {
        if( iter->parent().file().isEmpty() ) continue;
        _toggle( model_.index( *iter ) );
    }

}

//_______________________________________________
void IncludeFrame::_toggle( const QModelIndex& index )
{
    Debug::Throw( "IncludeFrame::_toggle.\n" );

    // check index validity, parent, and column
    if( !index.isValid() ) return;
    if( !model_.parent( index ).isValid() ) return;
    if( index.column() != IncludeFileModel::Selection ) return;

    // convert to File
    IncludeFile includeFile( model_.get( index ) );
    File this_File( includeFile.file() );

    // retrieve state
    // it is negated because it is supposibly the new state after the button being clicked
    bool checked( !includeFile.isSelected() );

    // check parent and retrieve associated File
    File parent_File( model_.get( model_.parent( index ) ).file() );

    // read text
    QFile in( parent_File );
    if( !in.open( QIODevice::ReadOnly ) ) return;
    QString text( in.readAll() );
    in.close();

    // split by lines
    QStringList lines( text.split( "\n" ) );

    bool found( false );
    bool enabled( false );
    for( QStringList::iterator iter = lines.begin(); iter != lines.end() && !found; ++iter )
    {

        // get position of regexp in line.
        int position( 0 );
        while( ( position = includeRegexp_.indexIn( *iter, position ) ) >= 0 && !found )
        {

            // see if found File match
            File includeFile( includeRegexp_.cap(2) );
            if( includeRegexp_.cap(1) == "bibliography" )  includeFile = TexString( includeFile ).addExtension( ".bib" );
            else includeFile = TexString( includeFile ).addExtension( ".tex" );

            if( !includeFile.isAbsolute() ) includeFile = includeFile.addPath( this_File.path() );
            if( includeFile == this_File )
            {

                Debug::Throw() << "IncludeFrame::_toggle - found File: " << includeFile << endl;

                // see if line is commented
                found = true;
                if( commentRegexp_.indexIn( *iter ) >= 0 )
                {
                    iter->replace( commentRegexp_, commentRegexp_.cap(1) );
                } else {
                    enabled = true;
                    *iter = "%"+*iter;
                }
            }

            position += includeRegexp_.matchedLength();
        }
    }

    // check if File was found
    if( !found )
    {
        QString buffer;
        QTextStream( &buffer ) << "unable to find include matching File " << this_File;
        InformationDialog( this, buffer ).exec();
        return;
    }

    // update item
    includeFile.setSelected( checked );
    model_.add( includeFile );

    // check if state has changed
    if( enabled == checked ) { return; }

    // make a backup of the input File
    parent_File.backup();

    // create output stream
    QFile out( parent_File );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    // write lines
    for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
    { out.write( QString( (*iter)+"\n" ).toAscii() ); }

    out.close();

    return;
}

//______________________________________________________________________
void IncludeFrame::_updateButtons( void )
{
    Debug::Throw( "IncludeFrame::_updateButtons.\n" );

    // get all files
    IncludeFileModel::List include_files( model_.children() );

    bool hasCheckable( false );
    bool hasChecked( false );
    bool hasUnchecked( false );

    // loop over files
    for( IncludeFileModel::List::const_iterator iter = include_files.begin(); iter != include_files.end(); ++iter )
    {
        if( iter->parent().file().isEmpty() ) continue;

        hasCheckable = true;
        if( iter->isSelected() ) hasChecked = true;
        else hasUnchecked = true;

    }

    selectAllButton_->setEnabled( hasUnchecked );
    selectNoneButton_->setEnabled( hasChecked );
    invertButton_->setEnabled( hasCheckable );

    buttonWidget_->setVisible( hasCheckable );

    // selection
    bool hasSelection( !_list().selectionModel()->selectedRows().isEmpty() );
    editButton_->setEnabled( hasSelection );
    spellCheckButton_->setEnabled( hasSelection );

}

//_____________________________________________________________
void IncludeFrame::_parseFile( const IncludeFile& parent )
{

    Debug::Throw() <<"IncludeFrame::_parseFile - File: " << parent.file() << endl;

    // try open File
    File fullname( parent.file() );
    QFile in( fullname );
    if( !in.open( QIODevice::ReadOnly ) ) return;

    // retrieve text
    QString text( in.readAll() );
    in.close();
    QStringList lines( text.split( "\n" ) );
    for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
    {

        // get position of regexp in line.
        int position( 0 );
        while( ( position = includeRegexp_.indexIn( *iter, position ) ) >= 0 )
        {

            Debug::Throw() << "IncludeFrame::_parseFile - found match in line " << *iter << endl;

            // get found File
            File includeFile( includeRegexp_.cap(2) );
            if( includeRegexp_.cap(1) == "bibliography" )  includeFile = TexString( includeFile ).addExtension( ".bib" );
            else includeFile = TexString( includeFile ).addExtension( ".tex" );

            if( !includeFile.isAbsolute() ) includeFile = includeFile.addPath( parent.file().path() );

            // see if enabled
            bool enabled( commentRegexp_.indexIn( *iter ) < 0 );

            // add File and enabled state to the list
            IncludeFile current( includeFile );
            current.setParent( parent );
            current.setSelected( enabled );

            // check for duplicates
            if( includes_.find( current ) != includes_.end() )
            {

                QString buffer;
                QTextStream( &buffer ) << "Duplicated File found in include list: " << current.file() << endl;
                InformationDialog( this, buffer ).centerOnWidget( window() ).exec();
                position += includeRegexp_.matchedLength();

            } else {

                includes_.insert( current );
                position += includeRegexp_.matchedLength();
                _parseFile( current );

            }

        }

    }

    return;

}

//___________________________________________________________________________
bool IncludeFrame::_fileIsAfs( const QString& file ) const
{ return file.indexOf( "/afs" ) == 0; }
