// $Id: MainWindow.cpp,v 1.43 2012/01/04 10:51:14 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "MainWindow.h"

#include "AnimatedTabWidget.h"
#include "Application.h"
#include "ActionFrame.h"
#include "BaseIcons.h"
#include "BrowsedLineEditor.h"
#include "ButtonFrame.h"
#include "Command.h"
#include "FileDialog.h"
#include "FileList.h"
#include "FigureFrame.h"
#include "IconEngine.h"
#include "IncludeFrame.h"
#include "InformationDialog.h"
#include "LineEditor.h"
#include "LogFrame.h"
#include "LogWidget.h"
#include "Menu.h"
#include "RecentFilesMenu.h"
#include "Singleton.h"
#include "StatusBar.h"
#include "TextFormat.h"
#include "TexString.h"
#include "Util.h"

#include <QtGui/QApplication>

//_______________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent )
{

    _updateWindowTitle();
    setOptionName( "MAIN_WINDOW" );

    Debug::Throw( "MainWindow::MainWindow.\n" );
    QWidget *main = new QWidget( this );
    QVBoxLayout* mainLayout = new QVBoxLayout();
    main->setLayout( mainLayout );
    mainLayout->setMargin(0);
    mainLayout->setSpacing(2);
    setCentralWidget( main );

    top_ = new QWidget( main );
    mainLayout->addWidget( top_ );
    QVBoxLayout* topLayout = new QVBoxLayout();
    topLayout->setMargin(0);
    topLayout->setSpacing(2);
    top_->setLayout( topLayout );

    // label and file editor
    BrowsedLineEditor *browsed_editor = new BrowsedLineEditor( top_ );
    editor_ = &browsed_editor->editor();
    editor().setHasClearButton( true );
    topLayout->addWidget( browsed_editor );

    // horizontal box for action frame and button frame
    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    topLayout->addLayout( hLayout, 0 );

    hLayout->addWidget( actionFrame_ = new ActionFrame( top_ ) );
    hLayout->addWidget( buttonFrame_ = new ButtonFrame( top_ ) );

    // tab widget
    tabWidget_ = new AnimatedTabWidget( main );
    mainLayout->addWidget( tabWidget_, 1 );

    // add tabs to widget
    tabWidget_->addTab( logFrame_ = new LogFrame( tabWidget_ ), "&Log" );
    tabWidget_->addTab( includeFrame_ = new IncludeFrame( tabWidget_ ), "&Includes" );
    tabWidget_->addTab( figureFrame_ = new FigureFrame( tabWidget_ ), "&Figures" );

    // state frame
    setStatusBar( statusbar_ = new StatusBar( this ) );
    statusBar().addLabel( 1, true );
    statusBar().addClock();

    // actions
    _installActions();
    Application &application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // menu
    setMenuBar( menu_ = new Menu( this ) );

    // connections
    connect( &includeFrame(), SIGNAL( updated( IncludeFileModel::List ) ), &figureFrame(), SLOT( update( IncludeFileModel::List ) ) );
    connect( &buttonFrame().editButton(), SIGNAL( clicked() ), SLOT( editTex() ) );
    connect( Singleton::get().application(), SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    _updateConfiguration();
    tabWidgetHeight_ = tabWidget_->height();

}

//_____________________________________________________
void MainWindow::setFile( const File& file )
{
    editor().setText( file );
    includeFrame().update();

    // update title
    _updateWindowTitle( file );
}

//_____________________________________________________
bool MainWindow::checkFile( File file, bool quiet )
{

    Debug::Throw( "ButtonFrame::_checkFile.\n" );
    file = file.expand();

    if( !file.exists() )
    {
        if( !quiet )
        {
            QString buffer;
            QTextStream( &buffer ) << "file \"" << file << "\" does not exists. Aborted.";
            InformationDialog( this, buffer ).exec();
        }
        return false;
    }

    if( file.isDirectory() )
    {
        if( !quiet )
        {
            QString buffer;
            QTextStream( &buffer ) << "file \"" << file << "\" is a directory. Aborted.";
            InformationDialog( this, buffer ).exec();
        }
        return false;
    }

    return true;
}

//_____________________________________
void MainWindow::edit( const QString& extension )
{

    Debug::Throw( "MainWindow::edit.\n" );

    // check editor
    if( !XmlOptions::get().find( "TEXT_EDITOR" ) ) {
        InformationDialog( this, "qText editor not set." ).exec();
        return;
    }

    File file( MainWindow::file() );
    if( extension.size() ) file = TexString( file ).addExtension( extension );

    Debug::Throw() << "MainWindow::edit - file=" << file << endl;

    if( !checkFile( file ) ) return;

    // update display and run command
    Application& application( *Singleton::get().application<Application>() );
    application.busy();
    QString buffer;
    QTextStream( &buffer ) << "Running " << XmlOptions::get().raw( "TEXT_EDITOR" ) << " ...";
    statusBar().label().setText( buffer );
    ( Command( XmlOptions::get().raw( "TEXT_EDITOR" ) ) << file.localName() ).run( file.path() );

    application.idle();
    statusBar().label().clear();
    setFile( file );

}

//_____________________________________
void MainWindow::open( FileRecord record )
{
    Debug::Throw( "MainWindow::open.\n" );

    if( record.file().isEmpty() )
    {

        // create file dialog
        FileDialog dialog( this );
        if( !file().isEmpty() ) dialog.selectFile( file().path() );
        QString file( dialog.getFile() );
        if( file.isNull() ) return;
        else record = FileRecord( file );
    }

    Singleton::get().application<Application>()->recentFiles().add( record.file().truncatedName() );
    menu().recentFilesMenu().setCurrentFile( record.file().truncatedName() );
    logFrame().editor().clear();
    setFile( record.file() );

}

//_______________________________________________
void MainWindow::clean( void )
{
    Debug::Throw( "MainWindow::clean.\n" );

    logFrame().editor().append( "\n" );
    logFrame().editor().append( "Cleaning", FORMAT::BOLD|FORMAT::UNDERLINE );
    logFrame().editor().append( "\n" );
    for( TexString::ExtensionList::iterator iter = TexString::cleanExtensions().begin(); iter != TexString::cleanExtensions().end(); ++iter )
    {

        File file( TexString( MainWindow::file() ).addExtension( QString(".")+*iter ) );
        if( file.exists() ) {
            QString buffer;
            QTextStream( &buffer ) << "rm " << file.localName() << "\n";
            logFrame().editor().append( buffer );
            file.remove();
        }
    }

    return;
}

//_______________________________________________
void MainWindow::addLatexHeader( void )
{

    Debug::Throw( "MainWindow::addLatexHeader.\n" );

    // retrieve input file
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    File file( TexString( mainWindow.file() ).addExtension( ".tex" ) );
    if( !mainWindow.checkFile( file ) ) return;

    // make a backup
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) ) return;
    QString body = in.readAll();
    in.close();

    if( body.contains( "\\begin{document}" ) )
    {
        QString buffer;
        QTextStream( &buffer ) << "file \"" << file << "\" already contains header. Aborted.";
        InformationDialog( this, buffer ).exec();
        return;
    }

    // reopen file (for output)
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    out.write( TexString::header().toAscii() );
    out.write( body.toAscii() );
    out.write( TexString::trailer().toAscii() );
    out.close();
    return;

}

//_______________________________________________
void MainWindow::_toLatexAccents( void )
{

    Debug::Throw( "MainWindow::_toLatexAccents.\n" );

    // retrieve input file
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    File file( TexString( mainWindow.file() ).addExtension( ".tex" ) );
    if( !mainWindow.checkFile( file ) ) return;

    // make a backup
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) ) return;
    QString body = in.readAll();
    in.close();

    for( TexString::ConversionList::const_iterator iter = TexString::conversions().begin(); iter != TexString::conversions().end(); ++iter )
    { if( !iter->first.isEmpty() ) body = body.replace( iter->first, iter->second ); }

    // reopen file (for output)
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    out.write( body.toAscii() );
    out.close();

    return;

}

//_______________________________________________
void MainWindow::_toTextAccents( void )
{

    Debug::Throw( "MainWindow::_toTextAccents.\n" );

    // retrieve input file
    MainWindow& mainWindow( *static_cast<MainWindow*>( window() ) );
    File file( TexString( mainWindow.file() ).addExtension( ".tex" ) );
    if( !mainWindow.checkFile( file ) ) return;

    // make a backup
    QFile in( file );
    if( !in.open( QIODevice::ReadOnly ) ) return;
    QString body = in.readAll();
    in.close();

    for( TexString::ConversionList::const_iterator iter = TexString::conversions().begin(); iter != TexString::conversions().end(); ++iter )
    { if( !iter->second.isEmpty() ) body = body.replace( iter->second, iter->first ); }

    // reopen file (for output)
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) ) return;

    out.write( body.toAscii() );
    out.close();

    return;

}

//____________________________________
void MainWindow::enterEvent( QEvent *event )
{
    Debug::Throw( "MainWindow::enterEvent.\n" );
    if( includeFrame().filesModified() ) includeFrame().update();
    return BaseMainWindow::enterEvent( event );
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    BaseMainWindow::closeEvent( event );
    qApp->quit();
}


//_______________________________________
void MainWindow::_updateConfiguration( void )
{ Debug::Throw( "MainWindow::_updateConfiguration" ); }

//_______________________________________
void MainWindow::_toggleLogVisibility( bool state )
{

    Debug::Throw( "MainWindow::_toggleLogVisibility.\n" );
    if( !state )
    {

        tabWidgetHeight_ = tabWidget_->y() + tabWidget_->height() - (top_->y() + top_->height() );
        tabWidget_->setVisible( false );
        layout()->activate();
        resize( width(), height() - tabWidgetHeight_ );

        Debug::Throw() << "MainWindow::_toggleLogVisibility - tabWidgetHeight_: " << tabWidgetHeight_ << endl;

    } else {

        tabWidget_->setVisible( true );

    }

}


//_______________________________________
void MainWindow::_installActions( void )
{

    Debug::Throw( "MainWindow::_installActions.\n" );

    // open
    addAction( openAction_ = new QAction( IconEngine::get( ICONS::OPEN ), "Open ...", this ) );
    openAction_->setShortcut( Qt::CTRL + Qt::Key_O );
    connect( openAction_, SIGNAL( triggered() ), SLOT( open() ) );

    // show log
    addAction( logVisibilityAction_ = new QAction( "Show Information Panel", this ) );
    logVisibilityAction_->setShortcut( Qt::CTRL + Qt::Key_I );
    logVisibilityAction_->setCheckable( true );
    logVisibilityAction_->setChecked( true );
    connect( logVisibilityAction_, SIGNAL( toggled( bool ) ), SLOT( _toggleLogVisibility( bool ) ) );

    // convert accents
    addAction( toLatexAccentsAction_ = new QAction( "Text to LaTeX", this ) );
    connect( toLatexAccentsAction_, SIGNAL( triggered() ), SLOT( _toLatexAccents() ) );

    addAction( toTextAccentsAction_ = new QAction( "LaTeX to Text", this ) );
    connect( toTextAccentsAction_, SIGNAL( triggered() ), SLOT( _toTextAccents() ) );

}
