#ifndef _MainWindow_h_
#define _MainWindow_h_

// $Id: MainWindow.h,v 1.22 2012/01/04 10:31:24 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "AnimatedLineEditor.h"
#include "BaseMainWindow.h"
#include "Counter.h"
#include "Debug.h"
#include "FileRecord.h"
#include "WindowTitle.h"

#include <QtCore/QEvent>
#include <QtGui/QAction>
#include <cassert>

class ActionFrame;
class AnimatedTabWidget;
class ButtonFrame;
class FigureFrame;
class IncludeFrame;
class LogFrame;
class Menu;
class StatusBar;
class MainWindow;

/*!
\class MainWindow
\brief customized QVBox to recieve close event
*/
class MainWindow: public BaseMainWindow
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor;
    MainWindow( QWidget* parent );

    //! retrieve Menu singleton
    Menu &menu( void ) const
    { return *menu_; }

    //! retrieve editor
    AnimatedLineEditor &editor( void ) const
    { return *editor_; }

    //! retrieve ActionFrame
    ActionFrame &actionFrame( void ) const
    { return *actionFrame_; }

    //! retrieve ButtonFrame
    ButtonFrame &buttonFrame( void ) const
    { return *buttonFrame_; }

    //! retrieve LogFrame
    LogFrame &logFrame( void ) const
    { return *logFrame_; }

    //! retrieve IncludeFrame
    IncludeFrame &includeFrame( void ) const
    { return *includeFrame_; }

    //! retrieve FigureFrame
    FigureFrame &figureFrame( void ) const
    { return *figureFrame_; }


    //! retrieve StatusBar singleton
    StatusBar& statusBar( void ) const
    { return *statusbar_; }

    //! log visibility action
    QAction& logVisibilityAction( void ) const
    { return *logVisibilityAction_; }

    //! set file
    void setFile( const File& file );

    //! retrieve file from editor
    File file( void ) const
    { return editor().text(); }

    //! check file existence
    bool checkFile( File, bool quiet = false );

    //!@name actions
    //@{

    //! open action
    QAction& openAction( void ) const
    { return *openAction_; }

    //! to latex accent
    QAction& toLatexAccentsAction( void ) const
    { return *toLatexAccentsAction_; }

    //! to text accents
    QAction& toTextAccentsAction( void ) const
    { return *toTextAccentsAction_; }

    //@}

    public slots:

    //! edit file (with extension if any, row otherwise)
    void edit( const QString& = "" );

    //! edit tex file
    void open( FileRecord = FileRecord() );

    //! clean all useless latex files
    void clean( void );

    //! edit tex file
    void editTex( void )
    { edit( ".tex" ); }

    //! edit log file
    void editLog( void )
    { edit( ".log" ); }

    //! edit aux file
    void editAux( void )
    { edit( ".aux" ); }

    //! edit Toc file
    void editToc( void )
    { edit( ".toc" ); }

    //! edit Bib file
    void editBib( void )
    { edit( ".bib" ); }

    //! edit Tof file
    void editTof( void )
    { edit( ".tof" ); }

    //! edit ps file
    void editPS( void )
    { edit( ".ps" ); }

    //! add latex header/trailer to text file
    void addLatexHeader( void );

    signals:

    void resizeRequest( void );

    protected:

    //! close window event handler
    virtual void closeEvent( QCloseEvent* );

    //! enter
    virtual void enterEvent ( QEvent* );

    protected slots:

    //! toggle log visibility
    void _toggleLogVisibility( bool );

    //! update window title
    void _updateWindowTitle( const QString& text = QString() )
    { setWindowTitle( WindowTitle( text ) ); }

    //! replace accents (text to latex)
    void _toLatexAccents( void );

    //! replace accents (latex to text)
    void _toTextAccents( void );

    private slots:

    //! update configuration
    void _updateConfiguration();

    private:

    //! install actions
    void _installActions( void );

    //! top widget
    QWidget* top_;

    //! file line editor singleton
    AnimatedLineEditor* editor_;

    //! action frame singleton
    ActionFrame* actionFrame_;

    //! button frame singleton
    ButtonFrame* buttonFrame_;

    //!@name actions
    //@{

    //! open
    QAction* openAction_;

    //! to latex accents
    QAction* toLatexAccentsAction_;

    //! to text accents
    QAction* toTextAccentsAction_;

    //@}

    //! log visibility action
    /*! it is used to show/hide the entire tab widget */
    QAction* logVisibilityAction_;

    //! tab widget
    AnimatedTabWidget *tabWidget_;

    //! tab widget size
    int tabWidgetHeight_;

    //! log frame singleton
    LogFrame* logFrame_;

    //! includes frame singleton
    IncludeFrame* includeFrame_;

    //! figures frame singleton
    FigureFrame* figureFrame_;

    //! state frame singleton
    StatusBar* statusbar_;

    //! Menu singleton
    Menu* menu_;
};

#endif
