// $Id: Menu.cpp,v 1.39 2012/01/02 10:32:30 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Menu.h"

#include "Application.h"
#include "Config.h"
#include "CustomPixmap.h"
#include "DebugMenu.h"
#include "DefaultHelpText.h"
#include "File.h"
#include "HelpManager.h"
#include "HelpText.h"
#include "MainWindow.h"
#include "RecentFilesMenu.h"
#include "Singleton.h"
#include "TexString.h"
#include "XmlOptions.h"

#include <QtGui/QMenu>

//_______________________________________________
Menu::Menu( QWidget* parent ):
    QMenuBar( parent ),
    Counter( "Menu" ),
    thread_( this )
{
    Debug::Throw( "Menu::Menu.\n" );

    // generic menu
    QMenu *menu;

    Application &application( *Singleton::get().application<Application>() );
    MainWindow &mainwindow( *static_cast<MainWindow*>(window()) );

    // file menu
    menu = addMenu( "File" );
    menu->addAction( &mainwindow.openAction() );

    // open previous menu
    recentFilesMenu_ = new RecentFilesMenu( this, application.recentFiles() );
    connect( recentFilesMenu_, SIGNAL( fileSelected( FileRecord ) ), &mainwindow, SLOT( open( FileRecord ) ) );
    menu->addMenu( recentFilesMenu_ );

    // exit button
    menu->addSeparator();
    menu->addAction( &application.closeAction() );

    // extension submenu
    editionMenu_ = addMenu( "Edit" );
    connect( editionMenu_, SIGNAL( aboutToShow() ), this, SLOT( _updateEditionMenu() ) );

    // Tools menu
    menu = addMenu( "Tools" );
    menu->addAction( "Add latex Header", &mainwindow, SLOT( addLatexHeader( void ) ) );

    menu = menu->addMenu( "French Accents" );
    menu->addAction( &mainwindow.toLatexAccentsAction() );
    menu->addAction( &mainwindow.toTextAccentsAction() );

    // Settings menu
    menu = addMenu( "Settings" );
    menu->addAction( &mainwindow.logVisibilityAction() );
    menu->addSeparator();
    menu->addAction( &application.configurationAction() );

    // help manager
    BASE::HelpManager* help( new BASE::HelpManager( this ) );
    help->setWindowTitle( "Textool Handbook" );
    File help_file( XmlOptions::get().get<File>( "HELP_FILE" ) );
    if( help_file.exists() ) help->install( help_file );
    else
    {
        help->setFile( help_file );
        help->install( helpText );
        help->install( BASE::helpText, false );
    }

    // help menu
    menu = addMenu( "Help" );
    menu->addAction( &help->displayAction() );
    menu->addSeparator();
    menu->addAction( &application.aboutQtAction() );
    menu->addAction( &application.aboutAction() );

    // debug menu
    #ifdef DEBUG
    menu->addSeparator();
    DebugMenu *debug_menu( new DebugMenu( this ) );
    debug_menu->setTitle( "Debug" );
    debug_menu->addAction( &help->dumpAction() );
    menu->addMenu( debug_menu );
    #endif

}

//_______________________________________________
void Menu::customEvent( QEvent* event )
{

    if( event->type() != ValidFileEvent::eventType() ) return QWidget::customEvent( event );

    ValidFileEvent* valid_file_event( static_cast<ValidFileEvent*>(event) );
    if( !valid_file_event ) return QWidget::customEvent( event );

    // set file records validity
    const FileRecord::List& records( valid_file_event->records() );
    for( ActionMap::iterator iter = editionActions_.begin(); iter != editionActions_.end(); ++iter )
    {
        FileRecord::List::const_iterator found = find_if(
            records.begin(),
            records.end(),
            FileRecord::SameFileFTor( iter->second.file() ) );
        if( found == records.end() ) continue;
        iter->first->setEnabled( found->isValid() );
    }

    return QWidget::customEvent( event );

}

//_________________________________________________
void Menu::_updateEditionMenu( void )
{
    Debug::Throw( "Menu::_updateEditionMenu.\n" );

    // clear actions
    editionMenu_->clear();
    editionActions_.clear();

    // retrieve current file
    MainWindow &mainwindow( *static_cast<MainWindow*>(window()) );
    File base( mainwindow.file() );

    // generic file
    File file;

    // generic action
    QAction* action;

    file = TexString( base ).addExtension( ".tex" );
    editionActions_.push_back( std::make_pair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) ) );
    connect( action, SIGNAL( triggered() ), &mainwindow, SLOT( editTex() ) );

    file = TexString( base ).addExtension( ".log" );
    editionActions_.push_back( std::make_pair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) ) );
    connect( action, SIGNAL( triggered() ), &mainwindow, SLOT( editLog() ) );

    file = TexString( base ).addExtension( ".aux" );
    editionActions_.push_back( std::make_pair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) ) );
    connect( action, SIGNAL( triggered() ), &mainwindow, SLOT( editAux() ) );

    file = TexString( base ).addExtension( ".toc" );
    editionActions_.push_back( std::make_pair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) ) );
    connect( action, SIGNAL( triggered() ), &mainwindow, SLOT( editToc() ) );

    file = TexString( base ).addExtension( ".tof" );
    editionActions_.push_back( std::make_pair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) ) );
    connect( action, SIGNAL( triggered() ), &mainwindow, SLOT( editTof() ) );

    file = TexString( base ).addExtension( ".bib" );
    editionActions_.push_back( std::make_pair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) ) );
    connect( action, SIGNAL( triggered() ), &mainwindow, SLOT( editBib() ) );

    file = TexString( base ).addExtension( ".ps" );
    editionActions_.push_back( std::make_pair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) ) );
    connect( action, SIGNAL( triggered() ), &mainwindow, SLOT( editPS() ) );

    // add actions to menu and store files
    FileRecord::List records;
    for( ActionMap::const_iterator iter = editionActions_.begin(); iter != editionActions_.end(); ++iter )
    {
        editionMenu_->addAction( iter->first );
        records.push_back( iter->second );
    }

    // additional actions
    editionMenu_->addSeparator();
    editionMenu_->addAction( "&Any", &mainwindow, SLOT( edit() ) );

    // initalize thread and run.
    if( thread_.isRunning() ) return;
    thread_.setRecords( records );
    thread_.start();

}
